@@ -5,21 +5,23 @@ import re
 import sys
 import unittest
 from email import message_from_string
+from enum import Enum, auto
 
+import docutils.frontend
 import docutils.parsers.rst
 import docutils.utils
-import docutils.frontend
+import pytest
+from _pytest.capture import CaptureFixture
 
 import piplicenses
-from piplicenses import (__pkgname__, create_parser, output_colored,
-                         create_licenses_table, get_output_fields, get_sortby,
-                         factory_styled_table_with_args, create_warn_string,
-                         find_license_from_classifier, create_output_string,
-                         select_license_by_source, save_if_needs,
-                         RULE_ALL, RULE_FRAME, RULE_HEADER, RULE_NONE,
-                         DEFAULT_OUTPUT_FIELDS, SYSTEM_PACKAGES,
-                         LICENSE_UNKNOWN)
-
+from piplicenses import (
+    DEFAULT_OUTPUT_FIELDS, LICENSE_UNKNOWN, RULE_ALL, RULE_FRAME,
+    RULE_HEADER, RULE_NONE, SYSTEM_PACKAGES, CompatibleArgumentParser,
+    FromArg, __pkgname__, create_licenses_table, create_output_string,
+    create_parser, create_warn_string, enum_key_to_value,
+    factory_styled_table_with_args, find_license_from_classifier,
+    get_output_fields, get_sortby, output_colored, save_if_needs,
+    select_license_by_source, value_to_enum_key)
 
 UNICODE_APPENDIX = ""
 with open('tests/fixtures/unicode_characters.txt', encoding='utf-8') as f:
@@ -35,7 +37,6 @@ def get_installed_distributions_mocked(*args, **kwargs):
 
 
 get_installed_distributions_orig = piplicenses.get_installed_distributions
-piplicenses.get_installed_distributions = get_installed_distributions_mocked
 
 
 class CommandLineTestCase(unittest.TestCase):
@@ -57,8 +58,8 @@ class TestGetLicenses(CommandLineTestCase):
 
         return pkg_name_columns
 
-    def _create_license_columns(self, table):
-        index = DEFAULT_OUTPUT_FIELDS.index('License')
+    def _create_license_columns(self, table, output_fields):
+        index = output_fields.index('License')
 
         # XXX: access to private API
         rows = copy.deepcopy(table._rows)
@@ -92,7 +93,8 @@ class TestGetLicenses(CommandLineTestCase):
         self.assertEqual(RULE_FRAME, table.hrules)
 
         output_fields = get_output_fields(args)
-        self.assertEqual(output_fields, list(DEFAULT_OUTPUT_FIELDS))
+        self.assertEqual(output_fields,
+                         list(DEFAULT_OUTPUT_FIELDS) + ['License'])
         self.assertNotIn('Author', output_fields)
         self.assertNotIn('URL', output_fields)
 
@@ -109,47 +111,72 @@ class TestGetLicenses(CommandLineTestCase):
     def test_from_meta(self):
         from_args = ['--from=meta']
         args = self.parser.parse_args(from_args)
-        table = create_licenses_table(args)
 
         output_fields = get_output_fields(args)
         self.assertIn('License', output_fields)
 
-        license_columns = self._create_license_columns(table)
-        license_notation_as_meta = 'BSD-3-Clause'
+        table = create_licenses_table(args, output_fields)
+        license_columns = self._create_license_columns(table, output_fields)
+        license_notation_as_meta = 'MIT'
         self.assertIn(license_notation_as_meta, license_columns)
 
     def test_from_classifier(self):
         from_args = ['--from=classifier']
         args = self.parser.parse_args(from_args)
-        table = create_licenses_table(args)
-
         output_fields = get_output_fields(args)
+        table = create_licenses_table(args, output_fields)
+
         self.assertIn('License', output_fields)
 
-        license_columns = self._create_license_columns(table)
+        license_columns = self._create_license_columns(table, output_fields)
         license_notation_as_classifier = 'MIT License'
         self.assertIn(license_notation_as_classifier, license_columns)
 
     def test_from_mixed(self):
         from_args = ['--from=mixed']
         args = self.parser.parse_args(from_args)
-        table = create_licenses_table(args)
-
         output_fields = get_output_fields(args)
+        table = create_licenses_table(args, output_fields)
+
         self.assertIn('License', output_fields)
 
-        license_columns = self._create_license_columns(table)
+        license_columns = self._create_license_columns(table, output_fields)
         # Depending on the condition "MIT" or "BSD" etc.
         license_notation_as_classifier = 'MIT License'
         self.assertIn(license_notation_as_classifier, license_columns)
 
+    def test_from_all(self):
+        from_args = ['--from=all']
+        args = self.parser.parse_args(from_args)
+        output_fields = get_output_fields(args)
+        table = create_licenses_table(args, output_fields)
+
+        self.assertIn('License-Metadata', output_fields)
+        self.assertIn('License-Classifier', output_fields)
+
+        index_license_meta = output_fields.index('License-Metadata')
+        license_meta = []
+        for row in table._rows:
+            license_meta.append(row[index_license_meta])
+
+        index_license_classifier = output_fields.index('License-Classifier')
+        license_classifier = []
+        for row in table._rows:
+            license_classifier.append(row[index_license_classifier])
+
+        for license in ('BSD', 'MIT', 'Apache 2.0'):
+            self.assertIn(license, license_meta)
+        for license in ('BSD License', 'MIT License',
+                        'Apache Software License'):
+            self.assertIn(license, license_classifier)
+
     def test_find_license_from_classifier(self):
         metadata = ('Metadata-Version: 2.0\r\n'
                     'Name: pip-licenses\r\n'
                     'Version: 1.0.0\r\n'
                     'Classifier: License :: OSI Approved :: MIT License\r\n')
         message = message_from_string(metadata)
-        self.assertEqual('MIT License',
+        self.assertEqual(['MIT License'],
                          find_license_from_classifier(message))
 
     def test_display_multiple_license_from_classifier(self):
@@ -162,9 +189,9 @@ class TestGetLicenses(CommandLineTestCase):
                     'Classifier: License :: OSI Approved :: MIT License\r\n'
                     'Classifier: License :: Public Domain\r\n')
         message = message_from_string(metadata)
-        self.assertEqual('GNU General Public License v3 (GPLv3), '
-                         'MIT License, '
-                         'Public Domain',
+        self.assertEqual(['GNU General Public License v3 (GPLv3)',
+                          'MIT License',
+                          'Public Domain'],
                          find_license_from_classifier(message))
 
     def test_not_found_license_from_classifier(self):
@@ -172,28 +199,28 @@ class TestGetLicenses(CommandLineTestCase):
                                   'Name: pip-licenses\r\n'
                                   'Version: 1.0.0\r\n')
         message = message_from_string(metadata_as_no_license)
-        self.assertEqual(LICENSE_UNKNOWN,
+        self.assertEqual([],
                          find_license_from_classifier(message))
 
     def test_select_license_by_source(self):
         self.assertEqual('MIT License',
-                         select_license_by_source('classifier',
-                                                  'MIT License',
+                         select_license_by_source(FromArg.CLASSIFIER,
+                                                  ['MIT License'],
                                                   'MIT'))
 
         self.assertEqual(LICENSE_UNKNOWN,
-                         select_license_by_source('classifier',
-                                                  LICENSE_UNKNOWN,
+                         select_license_by_source(FromArg.CLASSIFIER,
+                                                  [],
                                                   'MIT'))
 
         self.assertEqual('MIT License',
-                         select_license_by_source('mixed',
-                                                  'MIT License',
+                         select_license_by_source(FromArg.MIXED,
+                                                  ['MIT License'],
                                                   'MIT'))
 
         self.assertEqual('MIT',
-                         select_license_by_source('mixed',
-                                                  LICENSE_UNKNOWN,
+                         select_license_by_source(FromArg.MIXED,
+                                                  [],
                                                   'MIT'))
 
     def test_with_system(self):
@@ -311,6 +338,27 @@ class TestGetLicenses(CommandLineTestCase):
         pkg_name_columns = self._create_pkg_name_columns(table)
         self.assertNotIn(ignore_pkg_name, pkg_name_columns)
 
+    def test_with_packages(self):
+        pkg_name = 'py'
+        only_packages_args = ['--packages=' + pkg_name]
+        args = self.parser.parse_args(only_packages_args)
+        table = create_licenses_table(args)
+
+        pkg_name_columns = self._create_pkg_name_columns(table)
+        self.assertListEqual([pkg_name], pkg_name_columns)
+
+    def test_with_packages_with_system(self):
+        if 'PTable' in SYSTEM_PACKAGES:
+            pkg_name = 'PTable'
+        else:
+            pkg_name = 'prettytable'
+        only_packages_args = ['--packages=' + pkg_name, "--with-system"]
+        args = self.parser.parse_args(only_packages_args)
+        table = create_licenses_table(args)
+
+        pkg_name_columns = self._create_pkg_name_columns(table)
+        self.assertListEqual([pkg_name], pkg_name_columns)
+
     def test_order_name(self):
         order_name_args = ['--order=name']
         args = self.parser.parse_args(order_name_args)
@@ -378,6 +426,8 @@ class TestGetLicenses(CommandLineTestCase):
     @unittest.skipIf(sys.version_info < (3, 6, 0),
                      "To unsupport Python 3.5 in the near future")
     def test_format_rst_without_filter(self):
+        piplicenses.get_installed_distributions = \
+            get_installed_distributions_mocked
         format_rst_args = ['--format=rst']
         args = self.parser.parse_args(format_rst_args)
         table = create_licenses_table(args)
@@ -389,8 +439,12 @@ class TestGetLicenses(CommandLineTestCase):
         self.assertEqual(RULE_ALL, table.hrules)
         with self.assertRaises(docutils.utils.SystemMessage):
             self.check_rst(str(table))
+        piplicenses.get_installed_distributions = \
+            get_installed_distributions_orig
 
     def test_format_rst_default_filter(self):
+        piplicenses.get_installed_distributions = \
+            get_installed_distributions_mocked
         format_rst_args = ['--format=rst', '--filter-strings']
         args = self.parser.parse_args(format_rst_args)
         table = create_licenses_table(args)
@@ -401,6 +455,8 @@ class TestGetLicenses(CommandLineTestCase):
         self.assertEqual('+', table.junction_char)
         self.assertEqual(RULE_ALL, table.hrules)
         self.check_rst(str(table))
+        piplicenses.get_installed_distributions = \
+            get_installed_distributions_orig
 
     def test_format_confluence(self):
         format_confluence_args = ['--format=confluence']
@@ -506,25 +562,32 @@ class TestGetLicenses(CommandLineTestCase):
         self.assertTrue(actual.endswith('\033[0m'))
 
     def test_without_filter(self):
+        piplicenses.get_installed_distributions = \
+            get_installed_distributions_mocked
         args = self.parser.parse_args([])
         packages = list(piplicenses.get_packages(args))
         self.assertIn(UNICODE_APPENDIX, packages[-1]["name"])
+        piplicenses.get_installed_distributions = \
+            get_installed_distributions_orig
 
     def test_with_default_filter(self):
+        piplicenses.get_installed_distributions = \
+            get_installed_distributions_mocked
         args = self.parser.parse_args(["--filter-strings"])
         packages = list(piplicenses.get_packages(args))
+        piplicenses.get_installed_distributions = \
+            get_installed_distributions_orig
         self.assertNotIn(UNICODE_APPENDIX, packages[-1]["name"])
 
     def test_with_specified_filter(self):
+        piplicenses.get_installed_distributions = \
+            get_installed_distributions_mocked
         args = self.parser.parse_args(["--filter-strings",
                                        "--filter-code-page=ascii"])
         packages = list(piplicenses.get_packages(args))
         self.assertNotIn(UNICODE_APPENDIX, packages[-1]["summary"])
-
-    def test_invalid_code_page(self):
-        with self.assertRaises(SystemExit):
-            self.parser.parse_args(["--filter-strings",
-                                    "--filter-code-page=XXX"])
+        piplicenses.get_installed_distributions = \
+            get_installed_distributions_orig
 
 
 class MockStdStream(object):
@@ -589,7 +652,8 @@ def test_allow_only(monkeypatch):
         "MIT License, Mozilla Public License 2.0 (MPL 2.0)",
         "Apache Software License, BSD License",
         "Python Software Foundation License, MIT License",
-        "Public Domain, Python Software Foundation License, BSD License,"
+        "Public Domain, Python Software Foundation License, BSD License, "
+        "GNU General Public License (GPL)",
         "GNU General Public License (GPL)",
         "GNU Library or Lesser General Public License (LGPL)",
     )
@@ -623,3 +687,54 @@ def test_fail_on(monkeypatch):
     assert '' == mocked_stdout.printed
     assert 'fail-on license MIT License was found for ' \
            'package' in mocked_stderr.printed
+
+
+def test_enums():
+    class TestEnum(Enum):
+        PLAIN = P = auto()
+        JSON_LICENSE_FINDER = JLF = auto()
+
+    assert TestEnum.PLAIN == TestEnum.P
+    assert getattr(TestEnum, value_to_enum_key('jlf')) == \
+        TestEnum.JSON_LICENSE_FINDER
+    assert value_to_enum_key('jlf') == 'JLF'
+    assert value_to_enum_key('json-license-finder') == \
+        'JSON_LICENSE_FINDER'
+    assert enum_key_to_value(TestEnum.JSON_LICENSE_FINDER) == \
+        'json-license-finder'
+    assert enum_key_to_value(TestEnum.PLAIN) == 'plain'
+
+
+@pytest.fixture(scope='package')
+def parser():
+    return create_parser()
+
+
+def test_verify_args(
+        parser: CompatibleArgumentParser, capsys: CaptureFixture):
+    # --with-license-file missing
+    with pytest.raises(SystemExit) as ex:
+        parser.parse_args(['--no-license-path'])
+    capture = capsys.readouterr().err
+    for arg in ('--no-license-path', '--with-license-file'):
+        assert arg in capture
+
+    with pytest.raises(SystemExit) as ex:
+        parser.parse_args(['--with-notice-file'])
+    capture = capsys.readouterr().err
+    for arg in ('--with-notice-file', '--with-license-file'):
+        assert arg in capture
+
+    # --filter-strings missing
+    with pytest.raises(SystemExit) as ex:
+        parser.parse_args(['--filter-code-page=utf8'])
+    capture = capsys.readouterr().err
+    for arg in ('--filter-code-page', '--filter-strings'):
+        assert arg in capture
+
+    # invalid code-page
+    with pytest.raises(SystemExit) as ex:
+        parser.parse_args(['--filter-strings', '--filter-code-page=XX'])
+    capture = capsys.readouterr().err
+    for arg in ('invalid code', '--filter-code-page'):
+        assert arg in capture
